﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;

namespace AyaClock.WPF
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        // 多重起動禁止のロジックは
        //   http://akabeko.sakura.ne.jp/blog/2009/10/wpf-%E3%82%A2%E3%83%97%E3%83%AA%E3%81%AE%E8%B5%B7%E5%8B%95%E5%87%A6%E7%90%86/
        // を参考にした

        /// <summary>
        /// 多重起動禁止用mutex
        /// </summary>
        private static Mutex _mutex;

        /// <summary>
        /// Windows 10 Anniversary Update以降かどうかを判定
        /// </summary>
        /// <returns></returns>
        /// 
        private bool IsWin10AUorLater()
        {
            Version OSVer = Environment.OSVersion.Version;

            if (OSVer.Major > 10)
                return true;
            if (OSVer.Major == 10)
            {
                if (OSVer.Minor > 0)
                    return true;
                else
                    return (OSVer.Build >= 14393);
            }
            else
                return false;
        }

        private void Application_Startup(object sender, StartupEventArgs e)
        {
            // Windows 10 Anniversary Update以降のみサポート
            if (!IsWin10AUorLater())
            {
                _ = MessageBox.Show("AyaClock.WPF needs Windows 10 Anniversary Update or later", "AyaClock.WPF");
                this.Shutdown();
                return;
            }

            // 多重起動チェック
            App._mutex = new Mutex(false, "AyaClock.WPF2023.mutex");
            if (!App._mutex.WaitOne(0, false))
            {
                App._mutex.Close();
                App._mutex = null;
                _ = MessageBox.Show("AyaClock.WPF already exists", "AyaClock.WPF");
                this.Shutdown();
                return;
            }
        }

        private void Application_Exit(object sender, ExitEventArgs e)
        {
            if (App._mutex == null) { return; }

            // release mutex
            App._mutex.ReleaseMutex();
            App._mutex.Close();
            App._mutex = null;
        }
    }
}
